import Host from './host';
import { INSTANCE, INTERNAL, NATIVE_NODE } from '../constant';
/**
 * Base Component
 */

var BaseComponent = /*#__PURE__*/function () {
  function BaseComponent(element) {
    this.__currentElement = element;
  }

  var _proto = BaseComponent.prototype;

  _proto.__initComponent = function __initComponent(parent, parentInstance, context) {
    this._parent = parent;
    this.__parentInstance = parentInstance;
    this._context = context;
    this._mountID = Host.__mountID++;
  };

  _proto.__destoryComponent = function __destoryComponent() {
    if (process.env.NODE_ENV !== 'production') {
      Host.reconciler.unmountComponent(this);
    }

    this.__currentElement = this[NATIVE_NODE] = this._parent = this.__parentInstance = this._context = null;

    if (this[INSTANCE]) {
      this[INSTANCE] = this[INSTANCE][INTERNAL] = null;
    }
  };

  _proto.__mountComponent = function __mountComponent(parent, parentInstance, context, nativeNodeMounter) {
    this.__initComponent(parent, parentInstance, context);

    this.__mountNativeNode(nativeNodeMounter);

    if (process.env.NODE_ENV !== 'production') {
      Host.reconciler.mountComponent(this);
    }

    var instance = {};
    instance[INTERNAL] = this;
    return instance;
  };

  _proto.unmountComponent = function unmountComponent(shouldNotRemoveChild) {
    if (this[NATIVE_NODE] && !shouldNotRemoveChild) {
      Host.driver.removeChild(this[NATIVE_NODE], this._parent);
    }

    this.__destoryComponent();
  };

  _proto.__getName = function __getName() {
    var currentElement = this.__currentElement;
    var type = currentElement && currentElement.type;
    return type && type.displayName || type && type.name || type || // Native component's name is type
    currentElement;
  };

  _proto.__mountNativeNode = function __mountNativeNode(nativeNodeMounter) {
    var nativeNode = this.__getNativeNode();

    var parent = this._parent;

    if (nativeNodeMounter) {
      nativeNodeMounter(nativeNode, parent);
    } else {
      Host.driver.appendChild(nativeNode, parent);
    }
  };

  _proto.__getNativeNode = function __getNativeNode() {
    return this[NATIVE_NODE] == null ? this[NATIVE_NODE] = this.__createNativeNode() : this[NATIVE_NODE];
  };

  _proto.__getPublicInstance = function __getPublicInstance() {
    return this.__getNativeNode();
  };

  return BaseComponent;
}();

export { BaseComponent as default };