"use strict";

exports.__esModule = true;
exports.default = instantiateComponent;

var _host = _interopRequireDefault(require("./host"));

var _types = require("../types");

var _error = require("../error");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function instantiateComponent(element) {
  var instance;

  if ((0, _types.isPlainObject)(element) && element !== null && element.type) {
    // Special case string values
    if ((0, _types.isString)(element.type)) {
      instance = new _host.default.__Native(element);
    } else {
      instance = new _host.default.__Composite(element);
    }
  } else if ((0, _types.isString)(element) || (0, _types.isNumber)(element)) {
    instance = new _host.default.__Text(String(element));
  } else if ((0, _types.isArray)(element)) {
    instance = new _host.default.__Fragment(element);
  } else {
    if (!(element === undefined || (0, _types.isNull)(element) || element === false || element === true)) {
      if (process.env.NODE_ENV !== 'production') {
        (0, _error.throwError)('Invalid child type, expected types: Element instance, string, boolean, array, null, undefined.', element);
      } else {
        (0, _error.throwMinifiedWarn)(2, element);
      }
    }

    instance = new _host.default.__Empty(element);
  }

  return instance;
}